<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="1.0"
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:t="http://qifstandards.org/xsd/qif2">
  
  <xsl:output method="xml" indent="yes" />

  <xsl:include href="CheckLibrary.xsl"/>

  <xsl:variable name="CheckFormatParameters" select="document('CheckFormatParameters.xml')/CheckFormatParameters" />

  <!-- skip text content -->
  <xsl:template match="text()">
  </xsl:template>

  <!-- apply templates -->
  <xsl:template match="/t:QIFDocument">
    <xsl:element name="CheckFormatReport">
      <xsl:apply-templates />
    </xsl:element>
  </xsl:template>
  
  <!-- 2D/3D NURBS curves, the check: the number of control points = the number of knots - the curve order -->
  <xsl:template match="/t:QIFDocument/t:Product/t:GeometrySet/t:Curve12Set/t:Nurbs12/t:Nurbs12Core
                     | /t:QIFDocument/t:Product/t:GeometrySet/t:Curve13Set/t:Nurbs13/t:Nurbs13Core">
    <xsl:variable name="nCP">
      <xsl:if test="t:CPs">
        <xsl:value-of select="t:CPs/@N"/>
      </xsl:if>
      <xsl:if test="t:CPsBinary">
        <xsl:value-of select="t:CPsBinary/@N"/>
      </xsl:if>
    </xsl:variable>
    <xsl:if test="not($nCP = t:Knots/@N - t:Order)">
      <xsl:for-each select=".">
        <xsl:call-template name="error_node">
          <xsl:with-param name="report">
            Nurbs12Core/Nurbs13Core: id=<xsl:value-of select="../@id"/>, nCP(<xsl:value-of select="$nCP"/>) != nKt(<xsl:value-of select="t:Knots/@N"/>) - order(<xsl:value-of select="t:Order"/>)
          </xsl:with-param>
        </xsl:call-template>
      </xsl:for-each>
    </xsl:if>
    <xsl:apply-templates />
  </xsl:template>

  <!-- NURBS surface, the check: the number of control points = ('nKtU' - 'ordU')*('nKtV' - 'ordV') -->
  <xsl:template match="/t:QIFDocument/t:Product/t:GeometrySet/t:SurfaceSet/t:Nurbs23/t:Nurbs23Core">
    <xsl:variable name="nCP">
      <xsl:if test="t:CPs">
        <xsl:value-of select="t:CPs/@N"/>
      </xsl:if>
      <xsl:if test="t:CPsBinary">
        <xsl:value-of select="t:CPsBinary/@N"/>
      </xsl:if>
    </xsl:variable>
    <xsl:if test="not($nCP = (t:KnotsU/@N - t:OrderU)*(t:KnotsV/@N - t:OrderV))">
      <xsl:for-each select=".">
        <xsl:call-template name="error_node">
          <xsl:with-param name="report">
            Nurbs23Core: id=<xsl:value-of select="../@id"/>, nCP(<xsl:value-of select="$nCP"/>) != (nKtU(<xsl:value-of select="t:KnotsU/@N"/>) - ordU(<xsl:value-of select="t:OrderU"/>))*(nKtV(<xsl:value-of select="t:KnotsV/@N"/>) - ordV(<xsl:value-of select="t:OrderV"/>))
          </xsl:with-param>
        </xsl:call-template>
      </xsl:for-each>
    </xsl:if>
    <xsl:apply-templates />
  </xsl:template>
  
  <!-- Array of elements, the check: number of elements must be equal to a value specified in the attribute 'N' -->
  <xsl:template match="//t:*[@N and count(t:*) &gt; 0]">
    <xsl:if test="not(@N = count(t:*))">
      <xsl:for-each select=".">
        <xsl:call-template name="error_node">
          <xsl:with-param name="report">
            Array: N(<xsl:value-of select="@N"/>) not equal nElem(<xsl:value-of select="count(t:*)"/>)
          </xsl:with-param>
        </xsl:call-template>
      </xsl:for-each>
    </xsl:if>
    <xsl:apply-templates />
  </xsl:template>

  <!-- ArcCircular13Core/Normal: unit vectors -->
  <!--<xsl:template match="//t:ArcCircular13Core">
    <xsl:variable name="ToleranceNorm">
      <xsl:value-of select="$CheckFormatParameters/Check[@Name='UnitVector']/Parameter[@Name='ToleranceNorm']"/>
    </xsl:variable>
    <xsl:variable name="MinSquaredNorm">
      <xsl:value-of select="(1 - $ToleranceNorm)*(1 - $ToleranceNorm)"/>
    </xsl:variable>
    <xsl:variable name="MaxSquaredNorm">
      <xsl:value-of select="(1 + $ToleranceNorm)*(1 + $ToleranceNorm)"/>
    </xsl:variable>
    <xsl:variable name="ns">
      <xsl:value-of select="normalize-space(t:Normal)"/>
    </xsl:variable>
    <xsl:variable name="nx">
      <xsl:value-of select="substring-before($ns,' ')"/>
    </xsl:variable>
    <xsl:variable name="ny">
      <xsl:value-of select="substring-before(substring-after($ns,' '), ' ')"/>
    </xsl:variable>
    <xsl:variable name="nz">
      <xsl:value-of select="substring-after(substring-after($ns,' '), ' ')"/>
    </xsl:variable>
    <xsl:variable name="norm2">
      <xsl:value-of select="$nx*$nx + $ny*$ny + $nz*$nz"/>
    </xsl:variable>
    <xsl:if test="not(($norm2 &gt;= $MinSquaredNorm) and ($norm2 &lt;= $MaxSquaredNorm))">
      <xsl:for-each select=".">
        <xsl:call-template name="error_node">
          <xsl:with-param name="report">
            ArcCircular13Core: id=<xsl:value-of select="../@id"/>, Normal(<xsl:value-of select="$nx"/>,<xsl:value-of select="$ny"/>,<xsl:value-of select="$nz"/>), norm2(<xsl:value-of select="$norm2"/>) not in range [<xsl:value-of select="$MinSquaredNorm"/>, <xsl:value-of select="$MaxSquaredNorm"/>]
          </xsl:with-param>
        </xsl:call-template>
      </xsl:for-each>
    </xsl:if>
    <xsl:apply-templates />
  </xsl:template>-->

  <!-- TODO idMax -->
  <!-- TODO nurbs23: nWt == nCp -->
  <!-- TODO base64: check length -->
  <!-- TODO meshtriangle: nm size = vx size, nb size = tr.size -->
  <!-- TODO facemesh: colors size = triangles size, visible, hidden <= tr size -->
  <!-- TODO asmpath: check sequence of components -->

</xsl:stylesheet>
