<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="1.0"
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:t="http://qifstandards.org/xsd/qif2">

  <xsl:output method="xml" indent="yes" />

  <xsl:include href="CheckLibrary.xsl"/>

  <xsl:variable name="CheckQualityParameters" select="document('CheckQualityParameters.xml')/CheckQualityParameters" />
  
  <!-- skip text content -->
  <xsl:template match="text()">
  </xsl:template>

  <!-- apply templates -->
  <xsl:template match="/t:QIFDocument">
    <xsl:element name="CheckQualityReport">
      <xsl:apply-templates />
    </xsl:element>
  </xsl:template>

  <!-- Excessively high-degree Curve (G-CU-HD) -->
  <xsl:template match="/t:QIFDocument/t:Product/t:GeometrySet/t:SurfaceSet/t:Nurbs23/t:Nurbs23Core">
    <xsl:variable name="MaxDegree">
      <xsl:value-of select="$CheckQualityParameters/Check[@Name='G-CU-HD']/Parameter[@Name='MaxDegree']"/>
    </xsl:variable>
    <xsl:if test="not(((t:OrderU - 1) &lt;= $MaxDegree) and ((t:OrderV - 1) &lt;= $MaxDegree))">
      <xsl:for-each select=".">
        <xsl:call-template name="error_node">
          <xsl:with-param name="report">
            Excessively high-degree Curve (G-CU-HD): id=<xsl:value-of select="../@id"/>, ordU(<xsl:value-of select="t:OrderU"/>), ordV(<xsl:value-of select="t:OrderV"/>), MaxDegree(<xsl:value-of select="$MaxDegree"/>)
          </xsl:with-param>
        </xsl:call-template>
      </xsl:for-each>
    </xsl:if>
    <xsl:apply-templates />
  </xsl:template>

  <!-- Free edge (G-SH-FR) -->
  <!-- Over-used edge (G-SH-NM) -->
  <xsl:template match="/t:QIFDocument/t:Product/t:TopologySet/t:EdgeSet/t:Edge">
    <xsl:variable name="idEdge">
      <xsl:value-of select="@id"/>
    </xsl:variable>
    <xsl:variable name="nEdgeRef">
      <xsl:value-of select="count(//t:QIFDocument/t:Product/t:TopologySet/t:LoopSet/t:Loop/t:CoEdges/t:CoEdge[t:EdgeOriented/t:Id = $idEdge])"/>
    </xsl:variable>
    <xsl:if test="not($nEdgeRef = 2)">
      <xsl:for-each select=".">
        <xsl:call-template name="error_node">
          <xsl:with-param name="report">
            Free edge (G-SH-FR): id=<xsl:value-of select="@id"/>, nEdgeRef(<xsl:value-of select="$nEdgeRef"/>)
          </xsl:with-param>
        </xsl:call-template>
      </xsl:for-each>
    </xsl:if>
    <xsl:if test="not($nEdgeRef &lt;= 2)">
      <xsl:for-each select=".">
        <xsl:call-template name="error_node">
          <xsl:with-param name="report">
            Over-used edge (G-SH-NM): id=<xsl:value-of select="@id"/>, nEdgeRef(<xsl:value-of select="$nEdgeRef"/>)
          </xsl:with-param>
        </xsl:call-template>
      </xsl:for-each>
    </xsl:if>
    <xsl:apply-templates />
  </xsl:template>

  <!-- Fragmented curve (G-CU-FG) -->
  <xsl:template match="//t:Polyline13Core
                     | //t:Polyline12Core">
    <xsl:variable name="MaxNumSegments">
      <xsl:value-of select="$CheckQualityParameters/Check[@Name='G-CU-FG']/Parameter[@Name='MaxNumSegments']"/>
    </xsl:variable>
    <xsl:variable name="nSeg">
      <xsl:if test="t:Points">
        <xsl:value-of select="t:Points/@N"/>
      </xsl:if>
      <xsl:if test="t:PointsBinary">
        <xsl:value-of select="t:PointsBinary/@N"/>
      </xsl:if>
    </xsl:variable>
    <xsl:if test="not($nSeg &lt;= $MaxNumSegments)">
      <xsl:for-each select=".">
        <xsl:call-template name="error_node">
          <xsl:with-param name="report">
            Fragmented curve (G-CU-FG): id=<xsl:value-of select="../@id"/>, nSeg(<xsl:value-of select="$nSeg"/>) >= MaxNumSegments(<xsl:value-of select="$MaxNumSegments"/>)
          </xsl:with-param>
        </xsl:call-template>
      </xsl:for-each>
    </xsl:if>
    <xsl:apply-templates />
  </xsl:template>

  <!-- Tiny Curve or segment (G-CU-TI) -->
  <!--<xsl:template match="//t:Segment13Core[../@id = 11]">
    <xsl:variable name="Tolerance">
      <xsl:value-of select="$CheckQualityParameters/Check[@Name='G-CU-TI']/Parameter[@Name='Tolerance']"/>
    </xsl:variable>
    <xsl:variable name="Tolerance2">
      <xsl:value-of select="number($Tolerance*$Tolerance)"/>
    </xsl:variable>
    <xsl:variable name="a">
      <xsl:value-of select="normalize-space(t:StartPoint)"/>
    </xsl:variable>
    <xsl:variable name="ax">
      <xsl:value-of select="substring-before($a,' ')"/>
    </xsl:variable>
    <xsl:variable name="ay">
      <xsl:value-of select="substring-before(substring-after($a,' '), ' ')"/>
    </xsl:variable>
    <xsl:variable name="az">
      <xsl:value-of select="substring-after(substring-after($a,' '), ' ')"/>
    </xsl:variable>
    <xsl:variable name="b">
      <xsl:value-of select="normalize-space(t:EndPoint)"/>
    </xsl:variable>
    <xsl:variable name="bx">
      <xsl:value-of select="substring-before($b,' ')"/>
    </xsl:variable>
    <xsl:variable name="by">
      <xsl:value-of select="substring-before(substring-after($b,' '), ' ')"/>
    </xsl:variable>
    <xsl:variable name="bz">
      <xsl:value-of select="substring-after(substring-after($b,' '), ' ')"/>
    </xsl:variable>
    <xsl:variable name="dist2">
      <xsl:value-of select="number(($bx - $ax)*($bx - $ax) + ($by - $ay)*($by - $ay) + ($bz - $az)*($bz - $az))"/>
    </xsl:variable>
    <xsl:if test="not($dist2 &gt; $Tolerance2)">
      <xsl:for-each select=".">
        <xsl:call-template name="error_node">
          <xsl:with-param name="report">
            Tiny Curve or segment (G-CU-TI): id=<xsl:value-of select="../@id"/>, dist2(<xsl:value-of select="$dist2"/>) &lt;= <xsl:value-of select="$Tolerance2"/>
          </xsl:with-param>
        </xsl:call-template>
      </xsl:for-each>
    </xsl:if>
    <xsl:apply-templates />
  </xsl:template>-->
  
  <!-- TODO Over-used vertex (G-SH-OU) -->
  <!-- TODO Over-Used Face (G-FA-OU) -->
  <!-- TODO Missing Part (G-PT-MI) -->
  <!-- TODO Over-used vertex (G-SH-OU) -->
  <!-- TODO Non-Graphical Annotation (A-AN-NG) -->

</xsl:stylesheet>
