<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="1.0"
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:t="http://qifstandards.org/xsd/qif2">

  <xsl:output method="xml" indent="yes" />

  <xsl:include href="CheckLibrary.xsl"/>

  <!-- skip text content -->
  <xsl:template match="text()">
  </xsl:template>

  <!-- apply templates -->
  <xsl:template match="/t:QIFDocument">
    <xsl:element name="CheckSemanticReport">
      <xsl:apply-templates />
    </xsl:element>
  </xsl:template>

  <!-- FeatureNominal, the check: must be connected to a topology entity -->
  <xsl:template match="/t:QIFDocument/t:Features/t:FeatureNominals/t:*">
    <xsl:variable name="nTopo">
      <xsl:value-of select="count(t:EntityInternalIds/t:Id | t:EntityExternalIds/t:Id)"/>
    </xsl:variable>
    <xsl:if test="not($nTopo &gt; 0)">
      <xsl:for-each select=".">
        <xsl:call-template name="error_node">
          <xsl:with-param name="report">
            FeatureNominal not connected with topology: id=<xsl:value-of select="@id"/>, nTopo(<xsl:value-of select="$nTopo"/>)
          </xsl:with-param>
        </xsl:call-template>
      </xsl:for-each>
    </xsl:if>
    <xsl:apply-templates />
  </xsl:template>

  <!-- PositionCharacteristicDefinition, the check: if ToleranceValue = 0 then MaterialCondition must be MAXIMUM  -->
  <xsl:template match="/t:QIFDocument/t:Characteristics/t:CharacteristicDefinitions/t:PositionCharacteristicDefinition">
    <xsl:if test="not((t:ToleranceValue != 0) or (t:MaterialCondition = 'MAXIMUM'))">
      <xsl:for-each select=".">
        <xsl:call-template name="error_node">
          <xsl:with-param name="report">
            PositionCharacteristicDefinition: id=<xsl:value-of select="@id"/>, ToleranceValue=0 and MaterialCondition(<xsl:value-of select="t:MaterialCondition"/>) != 'MAXIMUM'
          </xsl:with-param>
        </xsl:call-template>
      </xsl:for-each>
    </xsl:if>
    <xsl:apply-templates />
  </xsl:template>
  
  <!-- PositionCharacteristicDefinition: if ZoneShape = DiametricalZone then  -->
  <!--<xsl:template match="/t:QIFDocument/t:Characteristics/t:CharacteristicNominals/t:PositionCharacteristicNominal">
    <xsl:variable name="idDefinition">
      <xsl:value-of select="t:CharacteristicDefinitionId"/>
    </xsl:variable>
    <xsl:variable name="ZoneShape">
      <xsl:value-of select="name(/t:QIFDocument/t:Characteristics/t:CharacteristicDefinitions/t:PositionCharacteristicDefinition[@id = $idDefinition]/t:ZoneShape/t:*[1])"/>
    </xsl:variable>
    <xsl:if test="1">
      <xsl:for-each select=".">
        <xsl:call-template name="error_node">
          <xsl:with-param name="report">
            PositionCharacteristicNominal: id=<xsl:value-of select="@id"/>, idDefinition(<xsl:value-of select="$idDefinition"/>), ZoneShape(<xsl:value-of select="$ZoneShape"/>)
          </xsl:with-param>
        </xsl:call-template>
      </xsl:for-each>
    </xsl:if>
    <xsl:apply-templates />
  </xsl:template>-->

  <!-- TODO CN Flatness -> FN Cylinder -->
  <!-- TODO CN without topology -->

</xsl:stylesheet>
